@@ -49,16 +49,14 @@ int waifu2x_getData(void*& out, unsigned long& outSize, double& tick, int& callB
     outSize = v.outSize;
 
     v.out = NULL;
-    clock_t encodeTick = v.encodeTick - v.startTick;
-    clock_t procTick = v.procTick - v.encodeTick;
-    clock_t decodeTick = v.saveTick - v.procTick;
-    clock_t allTick = v.saveTick - v.startTick;
+    double encodeTick = (v.encodeTick.time + v.encodeTick.millitm/1000.0) - (v.startTick.time + v.encodeTick.millitm / 1000.0);
+    double procTick = (v.procTick.time + v.procTick.millitm / 1000.0) - (v.encodeTick.time + v.encodeTick.millitm / 1000.0);
+    double decodeTick = (v.saveTick.time + v.saveTick.millitm / 1000.0) - (v.procTick.time + v.procTick.millitm/1000.0);
+    double allTick = (v.saveTick.time + v.saveTick.millitm / 1000.0) - (v.startTick.time + v.startTick.millitm / 10000.0);
 
-    waifu2x_printf(stdout, "[waifu2x] end encode imageId :%d, encode:%f, proc:%f, decode:%f, \n",
-        v.callBack, ((double)encodeTick) / CLOCKS_PER_SEC,
-        ((double)procTick) / CLOCKS_PER_SEC,
-        ((double)decodeTick) / CLOCKS_PER_SEC);
-    tick = ((double)allTick) / CLOCKS_PER_SEC;
+    waifu2x_printf(stdout, "[waifu2x] end encode imageId :%d, encode:%.2fs, proc:%.2fs, decode:%.2fs, \n",
+        v.callBack, encodeTick, procTick, decodeTick);
+    tick = allTick;
     return v.id;
 }
 
@@ -80,8 +78,7 @@ void* waifu2x_proc(void* args)
         {
             waifu2x = Waifu2xList[v.modelIndex];
         }
-        
-        v.startTick = clock();
+        ftime(&v.startTick);
         unsigned char* pixeldata = 0;
         int w;
         int h;
@@ -120,8 +117,8 @@ void* waifu2x_proc(void* args)
         else
             name = "cpu";
 
-        waifu2x_printf(stdout, "[waifu2x] start encode imageId :%d, gpu:%s, format:%s, model:%s, noise:%d, scale:%d, tta:%d\n",
-            v.callBack, name, v.file.c_str(), waifu2x->mode_name.c_str(), waifu2x->noise, waifu2x->scale, waifu2x->tta_mode);
+        waifu2x_printf(stdout, "[waifu2x] start encode imageId :%d, gpu:%s, format:%s, model:%s, noise:%d, scale:%d, tta:%d, tileSize:%d\n",
+            v.callBack, name, v.file.c_str(), waifu2x->mode_name.c_str(), waifu2x->noise, waifu2x->scale, waifu2x->tta_mode, v.tileSize);
 
         if (waifu2x && pixeldata)
         {
@@ -136,7 +133,7 @@ void* waifu2x_proc(void* args)
             //{
             //    v.file = "png";
             //}
-            v.encodeTick = clock();
+            ftime(&v.encodeTick);
             //waifu2x->process(v.inimage, v.outimage);
             int scale_run_count = 1;
             if (v.toH > 0 && v.toW > 0 && h > 0 && w > 0)
@@ -150,14 +147,18 @@ void* waifu2x_proc(void* args)
             int toW = w * pow(waifu2x->scale, scale_run_count);
             int toH = h * pow(waifu2x->scale, scale_run_count);
             v.outimage = ncnn::Mat(toW, toH, (size_t)c, c);
-
+            
+            if (waifu2x->scale <= 1)
+            {
+                scale_run_count = 1;
+            }
             for (int i = 0; i < scale_run_count; i++)
             {
                 if (i == scale_run_count - 1)
                 {
                     waifu2x_printf(stdout, "[waifu2x] start encode imageId :%d, count:%d, h:%d->%d, w:%d->%d \n",
                         v.callBack, i + 1, v.inimage.h, v.outimage.h, v.inimage.w, v.outimage.w);
-                    waifu2x->process(v.inimage, v.outimage);
+                    waifu2x->process(v.inimage, v.outimage, v.tileSize);
                     v.inimage.release();
                 }
                 else
@@ -165,7 +166,7 @@ void* waifu2x_proc(void* args)
                     ncnn::Mat tmpimage(v.inimage.w * 2, v.inimage.h * 2, (size_t)v.inimage.elemsize, (int)v.inimage.elemsize);
                     waifu2x_printf(stdout, "[waifu2x] start encode imageId :%d, count:%d, h:%d->%d, w:%d->%d \n",
                         v.callBack, i + 1, v.inimage.h, tmpimage.h, v.inimage.w, tmpimage.w);
-                    waifu2x->process(v.inimage, tmpimage);
+                    waifu2x->process(v.inimage, tmpimage, v.tileSize);
                     v.inimage.release();
                     v.inimage = tmpimage;
                 }
@@ -175,8 +176,7 @@ void* waifu2x_proc(void* args)
 //#else
             if (pixeldata) stbi_image_free(pixeldata);
 //#endif
-            v.procTick = clock();
-
+            ftime(&v.procTick);
             int success = 0;
             if (!v.file.compare("bmp") || !v.file.compare("BMP"))
             {
@@ -200,21 +200,6 @@ void* waifu2x_proc(void* args)
                     success = false;
                 }
             }
-            else if (!v.file.compare("gif") || !v.file.compare("GIF"))
-            {
-                WriteData data(v.toH, v.toW, v.outimage.elempack);
-                stbi_write_bmp_to_func((stbi_write_func*)write_jpg_to_mem, (void*)&data, v.toW, v.toH, v.outimage.elempack, (unsigned char*)v.outimage.data);
-                if (data.writeSize > 0)
-                {
-                    success = true;
-                    v.out = malloc(data.writeSize);
-                    v.outSize = data.writeSize;
-                    memcpy(v.out, data.data, v.outSize);
-                }
-                else {
-                    success = false;
-                }
-            }
             else if (!v.file.compare("png") || !v.file.compare("PNG"))
             {
                 /*#if _WIN32
@@ -270,7 +255,7 @@ void* waifu2x_proc(void* args)
                 const char* error = stbi_failure_reason();
                 waifu2x_printf(stderr, "[waifu2x] encode image %d failed, %s\n", v.id, error);
             }
-            v.saveTick = clock();
+            ftime(&v.saveTick);
         }
         else
         {
@@ -368,7 +353,7 @@ int waifu2x_addModel(const char* name, int scale2, int noise2, int tta_mode, int
             tilesize = 32;
     }
     struct stat buffer;
-
+    if (GpuId == -1) tilesize = 400;
 
     if (stat(parampath, &buffer) != 0)
     {
@@ -394,11 +379,11 @@ int waifu2x_addModel(const char* name, int scale2, int noise2, int tta_mode, int
     waifu->load(paramfullpath, modelfullpath);
     waifu->noise = noise2;
     waifu->scale = scale2;
-    if (GpuId == -1)
-    {
+    //if (GpuId == -1)
+    //{
         // cpu only
-        tilesize = 4000;
-    }
+        //tilesize = 4000;
+    //}
 
     waifu->tilesize = tilesize;
     waifu->prepadding = prepadding;
@@ -432,17 +417,14 @@ int waifu2x_init_path(const char* modelPath2)
 
 int waifu2x_init_set(int gpuId2, int threadNum)
 {
-    if (gpuId2 < -1 || gpuId2 >  2) { 
-        waifu2x_printf(stderr, "[waifu2x] gpuId error, gpuId2:%d, threadNum:%d \n", gpuId2, threadNum);
-        return -1; 
-    };
-    if (threadNum <= 0 || threadNum > 32) { return -1; };
+    if (threadNum <= 0 || threadNum > 128) { return -1; };
 
     int jobs_proc = threadNum;
 
-    if (gpuId2 == 0) gpuId2 = ncnn::get_default_gpu_index();
+    //if (gpuId2 == 0) gpuId2 = ncnn::get_default_gpu_index();
     int cpu_count = std::max(1, ncnn::get_cpu_count());
     int gpu_count = ncnn::get_gpu_count();
+    if (gpu_count == 0) gpuId2 = -1;
     if (gpuId2 < -1 || gpuId2 >= gpu_count)
     {
         waifu2x_printf(stderr, "[waifu2x] invalid gpu device\n");
@@ -452,7 +434,7 @@ int waifu2x_init_set(int gpuId2, int threadNum)
     if (gpuId2 == -1)
     {
         jobs_proc = std::min(jobs_proc, cpu_count);
-        TotalJobsProc += 1;
+        TotalJobsProc += jobs_proc;
     }
     else
     {
@@ -484,17 +466,10 @@ int waifu2x_init_set(int gpuId2, int threadNum)
     ProcThreads.resize(TotalJobsProc);
     {
         int total_jobs_proc_id = 0;
-        if (gpuId2 == -1)
+        for (int j = 0; j < jobs_proc; j++)
         {
             ProcThreads[total_jobs_proc_id++] = new ncnn::Thread(waifu2x_proc);
         }
-        else
-        {
-            for (int j = 0; j < jobs_proc; j++)
-            {
-                ProcThreads[total_jobs_proc_id++] = new ncnn::Thread(waifu2x_proc);
-            }
-        }
     }
     GpuId = gpuId2;
     //waifu2x_printf(stdout, "init success, threadNum:%d\n", TotalJobsProc);
@@ -543,7 +518,7 @@ int waifu2x_check_init_model(int initModel)
 }
 
 
-int waifu2x_addData(const unsigned char* data, unsigned int size, int callBack, int modelIndex, const char* format, unsigned long toW, unsigned long toH, float scale)
+int waifu2x_addData(const unsigned char* data, unsigned int size, int callBack, int modelIndex, const char* format, unsigned long toW, unsigned long toH, float scale, int tileSize)
 {
     Task v;
     TaskId ++;
@@ -555,6 +530,7 @@ int waifu2x_addData(const unsigned char* data, unsigned int size, int callBack,
     v.toH = toH;
     v.toW = toW;
     v.scale = scale;
+    v.tileSize = tileSize;
     if ((toH <= 0 || toW <= 0) && scale <= 0)
         return -1;
     int sts = waifu2x_check_init_model(modelIndex);
@@ -578,6 +554,9 @@ int waifu2x_stop()
             Toproc.put(end);
         }
 
+        Task end2;
+        end2.id = -233;
+        Tosave.put(end2);
         for (int i = 0; i < TotalJobsProc; i++)
         {
             ProcThreads[i]->join();
